# libguestfs Python bindings
# Copyright (C) 2017 Red Hat Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

import os
import sys
import shutil
import tempfile
import unittest

import guestfs


class Test830RHBZ1406906(unittest.TestCase):
    def setUp(self):
        self.tempdir = tempfile.mkdtemp()

    def tearDown(self):
        shutil.rmtree(self.tempdir)

    def test_rhbz1406906(self):
        g = guestfs.GuestFS(python_return_dict=True)

        g.add_drive_scratch(512 * 1024 * 1024)
        g.launch()

        g.part_disk("/dev/sda", "mbr")
        g.mkfs("ext4", "/dev/sda1")
        g.mount("/dev/sda1", "/")

        self.assertEqual(g.find("/"), ['lost+found'])

        # touch file with illegal unicode character
        non_utf8_fname = "\udcd4"
        open(os.path.join(self.tempdir, non_utf8_fname), "w").close()

        g.copy_in(self.tempdir, "/")

        if sys.version_info >= (3, 0):
            with self.assertRaises(UnicodeDecodeError):
                g.find("/")  # segfault here on Python 3
        elif sys.version_info >= (2, 0):
            self.assertTrue(
                any(path for path in g.find("/") if non_utf8_fname in path))



# libguestfs Python bindings
# Copyright (C) 2013-2020 Red Hat Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

# Test all the different return values.

import unittest
import guestfs
from .tests_helper import *


class Test090PythonRetValues(unittest.TestCase):
    def test_rint(self):
        g = guestfs.GuestFS()

        self.assertAlmostEqual(g.internal_test_rint("10"), 10, places=1)

        self.assertRaises(RuntimeError, g.internal_test_rinterr)

    def test_rint64(self):
        g = guestfs.GuestFS()

        self.assertAlmostEqual(g.internal_test_rint64("10"),
                               int_type(10), places=1)

        self.assertRaises(RuntimeError, g.internal_test_rint64err)

    def test_rbool(self):
        g = guestfs.GuestFS()

        self.assertTrue(g.internal_test_rbool("true"))
        self.assertFalse(g.internal_test_rbool("false"))

        self.assertRaises(RuntimeError, g.internal_test_rboolerr)

    def test_rconststring(self):
        g = guestfs.GuestFS()

        self.assertEqual(g.internal_test_rconststring("test"), "static string")

        self.assertRaises(RuntimeError, g.internal_test_rconststringerr)

    def test_rconstoptstring(self):
        g = guestfs.GuestFS()

        self.assertEqual(g.internal_test_rconstoptstring("test"),
                         "static string")

        # this never fails
        self.assertIsNone(g.internal_test_rconstoptstringerr())

    def test_rstring(self):
        g = guestfs.GuestFS()

        self.assertEqual(g.internal_test_rstring("test"), "test")

        self.assertRaises(RuntimeError, g.internal_test_rstringerr)

    def test_rstringlist(self):
        g = guestfs.GuestFS()

        self.assertEqual(g.internal_test_rstringlist("0"), [])
        self.assertEqual(g.internal_test_rstringlist("5"),
                         ["0", "1", "2", "3", "4"])

        self.assertRaises(RuntimeError, g.internal_test_rstringlisterr)

    def test_rstruct(self):
        g = guestfs.GuestFS()

        s = g.internal_test_rstruct("unused")
        self.assertIsInstance(s, dict)
        self.assertEqual(s["pv_name"], "pv0")

        self.assertRaises(RuntimeError, g.internal_test_rstructerr)

    def test_rstructlist(self):
        g = guestfs.GuestFS()

        self.assertEqual(g.internal_test_rstructlist("0"), [])
        retlist = g.internal_test_rstructlist("5")
        self.assertIsInstance(retlist, list)
        self.assertEqual(len(retlist), 5)
        for i in range(0, 5):
            self.assertIsInstance(retlist[i], dict)
            self.assertEqual(retlist[i]["pv_name"], "pv%d" % i)

        self.assertRaises(RuntimeError, g.internal_test_rstructlisterr)

    def test_rhashtable_list(self):
        g = guestfs.GuestFS(python_return_dict=False)

        self.assertEqual(g.internal_test_rhashtable("0"), [])
        r = g.internal_test_rhashtable("5")
        self.assertEqual(r, [("0", "0"), ("1", "1"), ("2", "2"),
                             ("3", "3"), ("4", "4")])

        self.assertRaises(RuntimeError, g.internal_test_rhashtableerr)

    def test_rhashtable_dict(self):
        g = guestfs.GuestFS(python_return_dict=True)

        self.assertEqual(g.internal_test_rhashtable("0"), {})
        r = g.internal_test_rhashtable("5")
        self.assertEqual(r, {"0": "0", "1": "1", "2": "2", "3": "3", "4": "4"})

        self.assertRaises(RuntimeError, g.internal_test_rhashtableerr)

    def test_rbufferout(self):
        g = guestfs.GuestFS()

        self.assertEqual(g.internal_test_rbufferout("test"), b'test')

        self.assertRaises(RuntimeError, g.internal_test_rbufferouterr)



# libguestfs Python bindings
# Copyright (C) 2011 Red Hat Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

# Test implicit vs explicit closes of the handle (RHBZ#717786).

import unittest
import guestfs

close_invoked = 0


def close_callback(ev, eh, buf, array):
    global close_invoked
    close_invoked += 1


class Test800ExplicitClose(unittest.TestCase):
    def test_explicit_close(self):
        g = guestfs.GuestFS(python_return_dict=True)

        g.close()               # explicit close
        del g                   # implicit close - should be no error/warning

        # Expect an exception if we call a method on a closed handle.
        g = guestfs.GuestFS(python_return_dict=True)
        g.close()
        self.assertRaises(guestfs.ClosedHandle, g.set_memsize, 512)
        del g

        # Verify that the handle is really being closed by g.close, by
        # setting up a close event and testing that it happened.
        g = guestfs.GuestFS(python_return_dict=True)

        g.set_event_callback(close_callback, guestfs.EVENT_CLOSE)

        self.assertEqual(close_invoked, 0)
        g.close()
        self.assertEqual(close_invoked, 1)

        del g
        self.assertEqual(close_invoked, 1)



# libguestfs Python bindings
# Copyright (C) 2009-2020 Red Hat Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

import unittest
import guestfs


class Test100Launch(unittest.TestCase):
    def test_launch(self):
        g = guestfs.GuestFS(python_return_dict=True)
        g.add_drive_scratch(500 * 1024 * 1024)
        g.launch()

        g.pvcreate("/dev/sda")
        g.vgcreate("VG", ["/dev/sda"])
        g.lvcreate("LV1", "VG", 200)
        g.lvcreate("LV2", "VG", 200)
        self.assertEqual(g.lvs(), ["/dev/VG/LV1", "/dev/VG/LV2"])
        g.shutdown()
        g.close()
